import os

from lib2to3 import refactor
from tabulate import tabulate

from .report import Report
from .record import Record, INFORMATION


class KodiRefactoringTool(refactor.RefactoringTool):

    def __init__(self, report, *args, **kwargs):
        self.report = report
        super(KodiRefactoringTool, self).__init__(*args, **kwargs)

    def short_path(self, path):
        return os.path.split(path)[1]

    def print_output(self, old, new, filepath, equal):
        """
        Called with the old version, new version, and filepath of a
        refactored file.

            :old: old text of refactored file
            :new: new text of refactored file
            :filepath: Path of the file
            :equal: Tells whether or not old is equal to new
        """

        self.headers = ['#', 'Existing Code', 'Changes required']
        self.table = []

        if equal:
            return

        required_changes = new.splitlines()
        existing_line = old.splitlines()

        for line in range(min(len(required_changes), len(existing_line))):
            if existing_line[line] != required_changes[line]:
                self.table.append([line + 1, existing_line[line], required_changes[line]])

        self.output = tabulate(self.table, headers=self.headers, tablefmt='pipe')
        self.report.add(Record(INFORMATION, self.short_path(filepath) + '\n' + self.output))


def Check_Py3_compatibility(report: Report, path: str):
    """
     Checks compatibility of addons with python3
        :path: path to the addon
    """
    fixer_names = []
    list_of_fixes = ['import', 'dict', 'except', 'filter', 'has_key', 'itertools',
                     'map', 'ne', 'next', 'print', 'renames', 'types', 'xrange', 'zip']

    for fix in list_of_fixes:
        fixer_names.append('lib2to3.fixes.fix_' + fix)

    rt = KodiRefactoringTool(report, fixer_names, options=None, explicit=None)
    rt.refactor([path])



import os
import logging
import xml.etree.ElementTree as ET
from distutils.version import LooseVersion
from .report import Report
from .record import PROBLEM, Record, INFORMATION

LOGGER = logging.getLogger(__name__)


def check_for_existing_addon(report: Report, addon_path: str, all_repo_addons: dict, pr: bool):
    """Check if addon submitted already exists or not
        :addon_path: path of the addon
        :all_repo_addons: dictionary return by all_repo_addon() function
    """

    addon_xml = os.path.join(addon_path, "addon.xml")
    addon_name, addon_version = _get_addon_name(addon_xml)

    for branch in sorted(all_repo_addons):
        repo_addons = all_repo_addons[branch]

        if addon_name in repo_addons:
            _check_versions(report, addon_name, branch, addon_version, repo_addons[addon_name], pr)
            return

    report.add(Record(INFORMATION, "This is a new addon"))


def _get_addon_name(xml_path: str):
    """returns name and version of the addon
        :xml_path: path of the xml file
    """

    tree = ET.parse(xml_path).getroot()
    return (tree.get("id"), tree.get("version"))


def _check_versions(report: Report, addon_name, branch, addon_version, repo_addons_version, pr):
    if pr:
        if LooseVersion(addon_version) > LooseVersion(repo_addons_version):
            LOGGER.info("%s addon have greater version: %s than repo_version: %s in branch %s"
                        % (addon_name, addon_version, repo_addons_version, branch))
        else:
            report.add(Record(PROBLEM, "%s addon already exists with version: %s in %s branch"
                              % (addon_name, repo_addons_version, branch)))
    else:
        if LooseVersion(addon_version) < LooseVersion(repo_addons_version):
            report.add(Record(PROBLEM, "%s addon already exist with version: %s in %s branch"
                              % (addon_name, repo_addons_version, branch)))
        else:
            report.add(Record(INFORMATION, "%s addon also exists in %s branch but with version: %s"
                              % (addon_name, branch, repo_addons_version)))



import os

from .report import Report
from .common import relative_path
from . import handle_files
from .record import PROBLEM, Record, WARNING


def check_for_legacy_strings_xml(report: Report, addon_path: str):
    """Find for the string.xml file in addon which was used in old versions
        :addon_path: path of the addon
    """
    for file in handle_files.find_files_recursive("strings.xml", os.path.join(addon_path, "resources", "language")):
        report.add(
            Record(PROBLEM, "Found %s please migrate to strings.po." % relative_path(file)))


def find_blacklisted_strings(report: Report, addon_path: str, problems: list, warnings: list, file_types: list):
    """Find for any blacklisted strings in the addons files
        :addon_path: Path of theh addon
        :problems: List of all the strings that will cause problem being in an addon
        :warnings: List of all the strings that shouldn't be in addon
                        but doesn't cause any problem
        :file_type: List of the whitelisted files to look into
    """
    for result in handle_files.find_in_file(addon_path, problems, file_types):
        report.add(Record(PROBLEM, "Found blacklisted term %s in file %s:%s (%s)"
                          % (result["term"], result["searchfile"], result["linenumber"], result["line"])))

    for result in handle_files.find_in_file(addon_path, warnings, file_types):
        report.add(Record(WARNING, "Found blacklisted term %s in file %s:%s (%s)"
                          % (result["term"], result["searchfile"], result["linenumber"], result["line"])))



import os
import re
import logging
from PIL import Image
from .report import Report
from .common import has_transparency, relative_path
from .record import PROBLEM, Record, WARNING, INFORMATION

LOGGER = logging.getLogger(__name__)


def check_artwork(report: Report, addon_path: str, parsed_xml, file_index: list):
    """Checks for icon/fanart/screenshot
        :addon_path: path to the folder having addon files
        :parsed_xml: xml file i.e addon.xml
        :file_index: list having name and path of all the files in an addon
    """
    art_type = ['icon', 'fanart', 'screenshot']
    for image_type in art_type:
        _check_image_type(report, image_type, parsed_xml, addon_path)

    for file in file_index:
        if re.match(r"(?!fanart\.jpg|icon\.png).*\.(png|jpg|jpeg|gif)$", file["name"]) is not None:
            image_path = os.path.join(file["path"], file["name"])
            try:
                Image.open(image_path)
            except IOError:
                report.add(
                    Record(PROBLEM, "Could not open image, is the file corrupted ? %s" % relative_path(image_path)))


def _check_image_type(report: Report, image_type: str, parsed_xml, addon_path: str):
    """Check for whether the given image type exists or not if they do """

    fallback, images = _assests(image_type, parsed_xml, addon_path)

    for image in images:
        if image:
            filepath = os.path.join(addon_path, image)

            if os.path.isfile(filepath):
                report.add(Record(INFORMATION, "Image %s exists" % image_type))
                try:
                    im = Image.open(filepath)
                    width, height = im.size

                    if image_type == "icon":
                        _check_icon(report, im, width, height)

                    elif image_type == "fanart":
                        _check_fanart(report, width, height)
                    else:
                        # screenshots have no size definitions
                        if has_transparency(im):
                            report.add(Record(PROBLEM, "%s should be solid. It has transparency." % image))
                        LOGGER.info("Artwork was a screenshot")
                except IOError:
                    report.add(
                        Record(PROBLEM, "Could not open image, is the file corrupted? %s" % relative_path(filepath)))

            else:
                # if it's a fallback path addons.xml should still be able to
                # get build
                if fallback:
                    report.add(Record(INFORMATION, "You might want to add a %s" % image_type))
                # it's no fallback path, so building addons.xml will crash -
                # this is a problem ;)
                else:
                    report.add(
                        Record(PROBLEM, "%s does not exist at specified path." % image_type))
        else:
            report.add(
                Record(WARNING, "Empty image tag found for %s" % image_type))


def _assests(image_type: str, parsed_xml, addon_path: str):
    """"""
    images = [image.text for image in parsed_xml.findall("./extension/assets/" + image_type)]

    fallback = False

    if not images and image_type == "icon":
        fallback = True
        images.append('icon.png')
    elif not images and image_type == "fanart":
        skip_addon_types = [".module.", "metadata.", "context.", ".language."]
        for addon_type in skip_addon_types:
            if addon_type in addon_path:
                break
        else:
            fallback = True
            images.append('fanart.jpg')

    return fallback, images


def _check_icon(report: Report, im, width, height):
    if has_transparency(im):
        report.add(Record(PROBLEM, "Icon.png should be solid. It has transparency."))

    icon_sizes = [(256, 256), (512, 512)]

    if (width, height) not in icon_sizes:
        report.add(Record(PROBLEM, "Icon should have either 256x256 or 512x512 but it has %sx%s" % (
            width, height)))
    else:
        report.add(
            Record(INFORMATION, "Icon dimensions are fine %sx%s" % (width, height)))


def _check_fanart(report: Report, width, height):
    fanart_sizes = [(1280, 720), (1920, 1080), (3840, 2160)]
    fanart_sizes_str = " or ".join(["%dx%d" % (w, h) for w, h in fanart_sizes])

    if (width, height) not in fanart_sizes:
        report.add(Record(PROBLEM, "Fanart should have either %s but it has %sx%s" % (
            fanart_sizes_str, width, height)))
    else:
        report.add(Record(INFORMATION, "Fanart dimensions are fine %sx%s" % (width, height)))
